# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
#
# deny overwrite

from __future__ import annotations

from .model import *


class DescribeUsersResult(core.Gs2Result):
    items: List[User] = None
    next_page_token: str = None

    def with_items(self, items: List[User]) -> DescribeUsersResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeUsersResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeUsersResult]:
        if data is None:
            return None
        return DescribeUsersResult()\
            .with_items([
                User.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')) if data.get('items') else 0)
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items) if self.items else 0)
            ],
            "nextPageToken": self.next_page_token,
        }


class CreateUserResult(core.Gs2Result):
    item: User = None

    def with_item(self, item: User) -> CreateUserResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CreateUserResult]:
        if data is None:
            return None
        return CreateUserResult()\
            .with_item(User.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class UpdateUserResult(core.Gs2Result):
    item: User = None

    def with_item(self, item: User) -> UpdateUserResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[UpdateUserResult]:
        if data is None:
            return None
        return UpdateUserResult()\
            .with_item(User.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class GetUserResult(core.Gs2Result):
    item: User = None

    def with_item(self, item: User) -> GetUserResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetUserResult]:
        if data is None:
            return None
        return GetUserResult()\
            .with_item(User.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DeleteUserResult(core.Gs2Result):
    item: User = None

    def with_item(self, item: User) -> DeleteUserResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DeleteUserResult]:
        if data is None:
            return None
        return DeleteUserResult()\
            .with_item(User.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DescribeSecurityPoliciesResult(core.Gs2Result):
    items: List[SecurityPolicy] = None
    next_page_token: str = None

    def with_items(self, items: List[SecurityPolicy]) -> DescribeSecurityPoliciesResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeSecurityPoliciesResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeSecurityPoliciesResult]:
        if data is None:
            return None
        return DescribeSecurityPoliciesResult()\
            .with_items([
                SecurityPolicy.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')) if data.get('items') else 0)
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items) if self.items else 0)
            ],
            "nextPageToken": self.next_page_token,
        }


class DescribeCommonSecurityPoliciesResult(core.Gs2Result):
    items: List[SecurityPolicy] = None
    next_page_token: str = None

    def with_items(self, items: List[SecurityPolicy]) -> DescribeCommonSecurityPoliciesResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeCommonSecurityPoliciesResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeCommonSecurityPoliciesResult]:
        if data is None:
            return None
        return DescribeCommonSecurityPoliciesResult()\
            .with_items([
                SecurityPolicy.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')) if data.get('items') else 0)
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items) if self.items else 0)
            ],
            "nextPageToken": self.next_page_token,
        }


class CreateSecurityPolicyResult(core.Gs2Result):
    item: SecurityPolicy = None

    def with_item(self, item: SecurityPolicy) -> CreateSecurityPolicyResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CreateSecurityPolicyResult]:
        if data is None:
            return None
        return CreateSecurityPolicyResult()\
            .with_item(SecurityPolicy.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class UpdateSecurityPolicyResult(core.Gs2Result):
    item: SecurityPolicy = None

    def with_item(self, item: SecurityPolicy) -> UpdateSecurityPolicyResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[UpdateSecurityPolicyResult]:
        if data is None:
            return None
        return UpdateSecurityPolicyResult()\
            .with_item(SecurityPolicy.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class GetSecurityPolicyResult(core.Gs2Result):
    item: SecurityPolicy = None

    def with_item(self, item: SecurityPolicy) -> GetSecurityPolicyResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetSecurityPolicyResult]:
        if data is None:
            return None
        return GetSecurityPolicyResult()\
            .with_item(SecurityPolicy.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DeleteSecurityPolicyResult(core.Gs2Result):
    item: SecurityPolicy = None

    def with_item(self, item: SecurityPolicy) -> DeleteSecurityPolicyResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DeleteSecurityPolicyResult]:
        if data is None:
            return None
        return DeleteSecurityPolicyResult()\
            .with_item(SecurityPolicy.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DescribeIdentifiersResult(core.Gs2Result):
    items: List[Identifier] = None
    next_page_token: str = None

    def with_items(self, items: List[Identifier]) -> DescribeIdentifiersResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribeIdentifiersResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeIdentifiersResult]:
        if data is None:
            return None
        return DescribeIdentifiersResult()\
            .with_items([
                Identifier.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')) if data.get('items') else 0)
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items) if self.items else 0)
            ],
            "nextPageToken": self.next_page_token,
        }


class CreateIdentifierResult(core.Gs2Result):
    item: Identifier = None
    client_secret: str = None

    def with_item(self, item: Identifier) -> CreateIdentifierResult:
        self.item = item
        return self

    def with_client_secret(self, client_secret: str) -> CreateIdentifierResult:
        self.client_secret = client_secret
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CreateIdentifierResult]:
        if data is None:
            return None
        return CreateIdentifierResult()\
            .with_item(Identifier.from_dict(data.get('item')))\
            .with_client_secret(data.get('clientSecret'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
            "clientSecret": self.client_secret,
        }


class GetIdentifierResult(core.Gs2Result):
    item: Identifier = None

    def with_item(self, item: Identifier) -> GetIdentifierResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetIdentifierResult]:
        if data is None:
            return None
        return GetIdentifierResult()\
            .with_item(Identifier.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DeleteIdentifierResult(core.Gs2Result):
    item: Identifier = None

    def with_item(self, item: Identifier) -> DeleteIdentifierResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DeleteIdentifierResult]:
        if data is None:
            return None
        return DeleteIdentifierResult()\
            .with_item(Identifier.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DescribeAttachedGuardsResult(core.Gs2Result):
    items: List[str] = None

    def with_items(self, items: List[str]) -> DescribeAttachedGuardsResult:
        self.items = items
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeAttachedGuardsResult]:
        if data is None:
            return None
        return DescribeAttachedGuardsResult()\
            .with_items([
                data.get('items')[i]
                for i in range(len(data.get('items')) if data.get('items') else 0)
            ])

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": [
                self.items[i]
                for i in range(len(self.items) if self.items else 0)
            ],
        }


class AttachGuardResult(core.Gs2Result):
    items: List[str] = None

    def with_items(self, items: List[str]) -> AttachGuardResult:
        self.items = items
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[AttachGuardResult]:
        if data is None:
            return None
        return AttachGuardResult()\
            .with_items([
                data.get('items')[i]
                for i in range(len(data.get('items')) if data.get('items') else 0)
            ])

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": [
                self.items[i]
                for i in range(len(self.items) if self.items else 0)
            ],
        }


class DetachGuardResult(core.Gs2Result):
    items: List[str] = None

    def with_items(self, items: List[str]) -> DetachGuardResult:
        self.items = items
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DetachGuardResult]:
        if data is None:
            return None
        return DetachGuardResult()\
            .with_items([
                data.get('items')[i]
                for i in range(len(data.get('items')) if data.get('items') else 0)
            ])

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": [
                self.items[i]
                for i in range(len(self.items) if self.items else 0)
            ],
        }


class DescribePasswordsResult(core.Gs2Result):
    items: List[Password] = None
    next_page_token: str = None

    def with_items(self, items: List[Password]) -> DescribePasswordsResult:
        self.items = items
        return self

    def with_next_page_token(self, next_page_token: str) -> DescribePasswordsResult:
        self.next_page_token = next_page_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribePasswordsResult]:
        if data is None:
            return None
        return DescribePasswordsResult()\
            .with_items([
                Password.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')) if data.get('items') else 0)
            ])\
            .with_next_page_token(data.get('nextPageToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items) if self.items else 0)
            ],
            "nextPageToken": self.next_page_token,
        }


class CreatePasswordResult(core.Gs2Result):
    item: Password = None

    def with_item(self, item: Password) -> CreatePasswordResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CreatePasswordResult]:
        if data is None:
            return None
        return CreatePasswordResult()\
            .with_item(Password.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class GetPasswordResult(core.Gs2Result):
    item: Password = None

    def with_item(self, item: Password) -> GetPasswordResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetPasswordResult]:
        if data is None:
            return None
        return GetPasswordResult()\
            .with_item(Password.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class EnableMfaResult(core.Gs2Result):
    item: Password = None
    challenge_token: str = None

    def with_item(self, item: Password) -> EnableMfaResult:
        self.item = item
        return self

    def with_challenge_token(self, challenge_token: str) -> EnableMfaResult:
        self.challenge_token = challenge_token
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[EnableMfaResult]:
        if data is None:
            return None
        return EnableMfaResult()\
            .with_item(Password.from_dict(data.get('item')))\
            .with_challenge_token(data.get('challengeToken'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
            "challengeToken": self.challenge_token,
        }


class ChallengeMfaResult(core.Gs2Result):
    item: Password = None

    def with_item(self, item: Password) -> ChallengeMfaResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[ChallengeMfaResult]:
        if data is None:
            return None
        return ChallengeMfaResult()\
            .with_item(Password.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DisableMfaResult(core.Gs2Result):
    item: Password = None

    def with_item(self, item: Password) -> DisableMfaResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DisableMfaResult]:
        if data is None:
            return None
        return DisableMfaResult()\
            .with_item(Password.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class DeletePasswordResult(core.Gs2Result):
    item: Password = None

    def with_item(self, item: Password) -> DeletePasswordResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DeletePasswordResult]:
        if data is None:
            return None
        return DeletePasswordResult()\
            .with_item(Password.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


class GetHasSecurityPolicyResult(core.Gs2Result):
    items: List[SecurityPolicy] = None

    def with_items(self, items: List[SecurityPolicy]) -> GetHasSecurityPolicyResult:
        self.items = items
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetHasSecurityPolicyResult]:
        if data is None:
            return None
        return GetHasSecurityPolicyResult()\
            .with_items([
                SecurityPolicy.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')) if data.get('items') else 0)
            ])

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items) if self.items else 0)
            ],
        }


class AttachSecurityPolicyResult(core.Gs2Result):
    items: List[SecurityPolicy] = None

    def with_items(self, items: List[SecurityPolicy]) -> AttachSecurityPolicyResult:
        self.items = items
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[AttachSecurityPolicyResult]:
        if data is None:
            return None
        return AttachSecurityPolicyResult()\
            .with_items([
                SecurityPolicy.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')) if data.get('items') else 0)
            ])

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items) if self.items else 0)
            ],
        }


class DetachSecurityPolicyResult(core.Gs2Result):
    items: List[SecurityPolicy] = None

    def with_items(self, items: List[SecurityPolicy]) -> DetachSecurityPolicyResult:
        self.items = items
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DetachSecurityPolicyResult]:
        if data is None:
            return None
        return DetachSecurityPolicyResult()\
            .with_items([
                SecurityPolicy.from_dict(data.get('items')[i])
                for i in range(len(data.get('items')) if data.get('items') else 0)
            ])

    def to_dict(self) -> Dict[str, Any]:
        return {
            "items": [
                self.items[i].to_dict() if self.items[i] else None
                for i in range(len(self.items) if self.items else 0)
            ],
        }


class LoginResult(core.Gs2Result):
    access_token: str = None
    token_type: str = None
    expires_in: int = None
    owner_id: str = None

    def with_access_token(self, access_token: str) -> LoginResult:
        self.access_token = access_token
        return self

    def with_token_type(self, token_type: str) -> LoginResult:
        self.token_type = token_type
        return self

    def with_expires_in(self, expires_in: int) -> LoginResult:
        self.expires_in = expires_in
        return self

    def with_owner_id(self, owner_id: str) -> LoginResult:
        self.owner_id = owner_id
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[LoginResult]:
        if data is None:
            return None
        return LoginResult()\
            .with_access_token(data.get('access_token'))\
            .with_token_type(data.get('token_type'))\
            .with_expires_in(data.get('expires_in'))\
            .with_owner_id(data.get('owner_id'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "access_token": self.access_token,
            "token_type": self.token_type,
            "expires_in": self.expires_in,
            "owner_id": self.owner_id,
        }


class LoginByUserResult(core.Gs2Result):
    item: ProjectToken = None

    def with_item(self, item: ProjectToken) -> LoginByUserResult:
        self.item = item
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[LoginByUserResult]:
        if data is None:
            return None
        return LoginByUserResult()\
            .with_item(ProjectToken.from_dict(data.get('item')))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "item": self.item.to_dict() if self.item else None,
        }


# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

from __future__ import annotations

import re
from typing import *
from gs2 import core


class LogSetting(core.Gs2Model):
    logging_namespace_id: str = None

    def with_logging_namespace_id(self, logging_namespace_id: str) -> LogSetting:
        self.logging_namespace_id = logging_namespace_id
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[LogSetting]:
        if data is None:
            return None
        return LogSetting()\
            .with_logging_namespace_id(data.get('loggingNamespaceId'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "loggingNamespaceId": self.logging_namespace_id,
        }


class FirebaseToken(core.Gs2Model):
    firebase_token_id: str = None
    user_id: str = None
    token: str = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_firebase_token_id(self, firebase_token_id: str) -> FirebaseToken:
        self.firebase_token_id = firebase_token_id
        return self

    def with_user_id(self, user_id: str) -> FirebaseToken:
        self.user_id = user_id
        return self

    def with_token(self, token: str) -> FirebaseToken:
        self.token = token
        return self

    def with_created_at(self, created_at: int) -> FirebaseToken:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> FirebaseToken:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> FirebaseToken:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        user_id,
    ):
        return 'grn:gs2:{region}:{ownerId}:gateway:{namespaceName}:user:{userId}:firebase:token'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            userId=user_id,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):gateway:(?P<namespaceName>.+):user:(?P<userId>.+):firebase:token', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):gateway:(?P<namespaceName>.+):user:(?P<userId>.+):firebase:token', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):gateway:(?P<namespaceName>.+):user:(?P<userId>.+):firebase:token', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_user_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):gateway:(?P<namespaceName>.+):user:(?P<userId>.+):firebase:token', grn)
        if match is None:
            return None
        return match.group('user_id')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[FirebaseToken]:
        if data is None:
            return None
        return FirebaseToken()\
            .with_firebase_token_id(data.get('firebaseTokenId'))\
            .with_user_id(data.get('userId'))\
            .with_token(data.get('token'))\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "firebaseTokenId": self.firebase_token_id,
            "userId": self.user_id,
            "token": self.token,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class WebSocketSession(core.Gs2Model):
    web_socket_session_id: str = None
    connection_id: str = None
    namespace_name: str = None
    user_id: str = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_web_socket_session_id(self, web_socket_session_id: str) -> WebSocketSession:
        self.web_socket_session_id = web_socket_session_id
        return self

    def with_connection_id(self, connection_id: str) -> WebSocketSession:
        self.connection_id = connection_id
        return self

    def with_namespace_name(self, namespace_name: str) -> WebSocketSession:
        self.namespace_name = namespace_name
        return self

    def with_user_id(self, user_id: str) -> WebSocketSession:
        self.user_id = user_id
        return self

    def with_created_at(self, created_at: int) -> WebSocketSession:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> WebSocketSession:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> WebSocketSession:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        user_id,
        connection_id,
    ):
        return 'grn:gs2:{region}:{ownerId}:gateway:{namespaceName}:user:{userId}:session:{connectionId}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            userId=user_id,
            connectionId=connection_id,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):gateway:(?P<namespaceName>.+):user:(?P<userId>.+):session:(?P<connectionId>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):gateway:(?P<namespaceName>.+):user:(?P<userId>.+):session:(?P<connectionId>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):gateway:(?P<namespaceName>.+):user:(?P<userId>.+):session:(?P<connectionId>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_user_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):gateway:(?P<namespaceName>.+):user:(?P<userId>.+):session:(?P<connectionId>.+)', grn)
        if match is None:
            return None
        return match.group('user_id')

    @classmethod
    def get_connection_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):gateway:(?P<namespaceName>.+):user:(?P<userId>.+):session:(?P<connectionId>.+)', grn)
        if match is None:
            return None
        return match.group('connection_id')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[WebSocketSession]:
        if data is None:
            return None
        return WebSocketSession()\
            .with_web_socket_session_id(data.get('webSocketSessionId'))\
            .with_connection_id(data.get('connectionId'))\
            .with_namespace_name(data.get('namespaceName'))\
            .with_user_id(data.get('userId'))\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "webSocketSessionId": self.web_socket_session_id,
            "connectionId": self.connection_id,
            "namespaceName": self.namespace_name,
            "userId": self.user_id,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class Namespace(core.Gs2Model):
    namespace_id: str = None
    name: str = None
    description: str = None
    firebase_secret: str = None
    log_setting: LogSetting = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_namespace_id(self, namespace_id: str) -> Namespace:
        self.namespace_id = namespace_id
        return self

    def with_name(self, name: str) -> Namespace:
        self.name = name
        return self

    def with_description(self, description: str) -> Namespace:
        self.description = description
        return self

    def with_firebase_secret(self, firebase_secret: str) -> Namespace:
        self.firebase_secret = firebase_secret
        return self

    def with_log_setting(self, log_setting: LogSetting) -> Namespace:
        self.log_setting = log_setting
        return self

    def with_created_at(self, created_at: int) -> Namespace:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> Namespace:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> Namespace:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:gateway:{namespaceName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):gateway:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):gateway:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):gateway:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Namespace]:
        if data is None:
            return None
        return Namespace()\
            .with_namespace_id(data.get('namespaceId'))\
            .with_name(data.get('name'))\
            .with_description(data.get('description'))\
            .with_firebase_secret(data.get('firebaseSecret'))\
            .with_log_setting(LogSetting.from_dict(data.get('logSetting')))\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "namespaceId": self.namespace_id,
            "name": self.name,
            "description": self.description,
            "firebaseSecret": self.firebase_secret,
            "logSetting": self.log_setting.to_dict() if self.log_setting else None,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

from __future__ import annotations

import re
from typing import *
from gs2 import core


class TransactionSetting(core.Gs2Model):
    enable_auto_run: bool = None
    enable_atomic_commit: bool = None
    transaction_use_distributor: bool = None
    acquire_action_use_job_queue: bool = None
    distributor_namespace_id: str = None
    key_id: str = None
    queue_namespace_id: str = None

    def with_enable_auto_run(self, enable_auto_run: bool) -> TransactionSetting:
        self.enable_auto_run = enable_auto_run
        return self

    def with_enable_atomic_commit(self, enable_atomic_commit: bool) -> TransactionSetting:
        self.enable_atomic_commit = enable_atomic_commit
        return self

    def with_transaction_use_distributor(self, transaction_use_distributor: bool) -> TransactionSetting:
        self.transaction_use_distributor = transaction_use_distributor
        return self

    def with_acquire_action_use_job_queue(self, acquire_action_use_job_queue: bool) -> TransactionSetting:
        self.acquire_action_use_job_queue = acquire_action_use_job_queue
        return self

    def with_distributor_namespace_id(self, distributor_namespace_id: str) -> TransactionSetting:
        self.distributor_namespace_id = distributor_namespace_id
        return self

    def with_key_id(self, key_id: str) -> TransactionSetting:
        self.key_id = key_id
        return self

    def with_queue_namespace_id(self, queue_namespace_id: str) -> TransactionSetting:
        self.queue_namespace_id = queue_namespace_id
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[TransactionSetting]:
        if data is None:
            return None
        return TransactionSetting()\
            .with_enable_auto_run(data.get('enableAutoRun'))\
            .with_enable_atomic_commit(data.get('enableAtomicCommit'))\
            .with_transaction_use_distributor(data.get('transactionUseDistributor'))\
            .with_acquire_action_use_job_queue(data.get('acquireActionUseJobQueue'))\
            .with_distributor_namespace_id(data.get('distributorNamespaceId'))\
            .with_key_id(data.get('keyId'))\
            .with_queue_namespace_id(data.get('queueNamespaceId'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "enableAutoRun": self.enable_auto_run,
            "enableAtomicCommit": self.enable_atomic_commit,
            "transactionUseDistributor": self.transaction_use_distributor,
            "acquireActionUseJobQueue": self.acquire_action_use_job_queue,
            "distributorNamespaceId": self.distributor_namespace_id,
            "keyId": self.key_id,
            "queueNamespaceId": self.queue_namespace_id,
        }


class LogSetting(core.Gs2Model):
    logging_namespace_id: str = None

    def with_logging_namespace_id(self, logging_namespace_id: str) -> LogSetting:
        self.logging_namespace_id = logging_namespace_id
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[LogSetting]:
        if data is None:
            return None
        return LogSetting()\
            .with_logging_namespace_id(data.get('loggingNamespaceId'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "loggingNamespaceId": self.logging_namespace_id,
        }


class ScriptSetting(core.Gs2Model):
    trigger_script_id: str = None
    done_trigger_target_type: str = None
    done_trigger_script_id: str = None
    done_trigger_queue_namespace_id: str = None

    def with_trigger_script_id(self, trigger_script_id: str) -> ScriptSetting:
        self.trigger_script_id = trigger_script_id
        return self

    def with_done_trigger_target_type(self, done_trigger_target_type: str) -> ScriptSetting:
        self.done_trigger_target_type = done_trigger_target_type
        return self

    def with_done_trigger_script_id(self, done_trigger_script_id: str) -> ScriptSetting:
        self.done_trigger_script_id = done_trigger_script_id
        return self

    def with_done_trigger_queue_namespace_id(self, done_trigger_queue_namespace_id: str) -> ScriptSetting:
        self.done_trigger_queue_namespace_id = done_trigger_queue_namespace_id
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[ScriptSetting]:
        if data is None:
            return None
        return ScriptSetting()\
            .with_trigger_script_id(data.get('triggerScriptId'))\
            .with_done_trigger_target_type(data.get('doneTriggerTargetType'))\
            .with_done_trigger_script_id(data.get('doneTriggerScriptId'))\
            .with_done_trigger_queue_namespace_id(data.get('doneTriggerQueueNamespaceId'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "triggerScriptId": self.trigger_script_id,
            "doneTriggerTargetType": self.done_trigger_target_type,
            "doneTriggerScriptId": self.done_trigger_script_id,
            "doneTriggerQueueNamespaceId": self.done_trigger_queue_namespace_id,
        }


class GitHubCheckoutSetting(core.Gs2Model):
    api_key_id: str = None
    repository_name: str = None
    source_path: str = None
    reference_type: str = None
    commit_hash: str = None
    branch_name: str = None
    tag_name: str = None

    def with_api_key_id(self, api_key_id: str) -> GitHubCheckoutSetting:
        self.api_key_id = api_key_id
        return self

    def with_repository_name(self, repository_name: str) -> GitHubCheckoutSetting:
        self.repository_name = repository_name
        return self

    def with_source_path(self, source_path: str) -> GitHubCheckoutSetting:
        self.source_path = source_path
        return self

    def with_reference_type(self, reference_type: str) -> GitHubCheckoutSetting:
        self.reference_type = reference_type
        return self

    def with_commit_hash(self, commit_hash: str) -> GitHubCheckoutSetting:
        self.commit_hash = commit_hash
        return self

    def with_branch_name(self, branch_name: str) -> GitHubCheckoutSetting:
        self.branch_name = branch_name
        return self

    def with_tag_name(self, tag_name: str) -> GitHubCheckoutSetting:
        self.tag_name = tag_name
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GitHubCheckoutSetting]:
        if data is None:
            return None
        return GitHubCheckoutSetting()\
            .with_api_key_id(data.get('apiKeyId'))\
            .with_repository_name(data.get('repositoryName'))\
            .with_source_path(data.get('sourcePath'))\
            .with_reference_type(data.get('referenceType'))\
            .with_commit_hash(data.get('commitHash'))\
            .with_branch_name(data.get('branchName'))\
            .with_tag_name(data.get('tagName'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "apiKeyId": self.api_key_id,
            "repositoryName": self.repository_name,
            "sourcePath": self.source_path,
            "referenceType": self.reference_type,
            "commitHash": self.commit_hash,
            "branchName": self.branch_name,
            "tagName": self.tag_name,
        }


class Config(core.Gs2Model):
    key: str = None
    value: str = None

    def with_key(self, key: str) -> Config:
        self.key = key
        return self

    def with_value(self, value: str) -> Config:
        self.value = value
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Config]:
        if data is None:
            return None
        return Config()\
            .with_key(data.get('key'))\
            .with_value(data.get('value'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "key": self.key,
            "value": self.value,
        }


class RarityParameterValue(core.Gs2Model):
    name: str = None
    resource_name: str = None
    resource_value: int = None

    def with_name(self, name: str) -> RarityParameterValue:
        self.name = name
        return self

    def with_resource_name(self, resource_name: str) -> RarityParameterValue:
        self.resource_name = resource_name
        return self

    def with_resource_value(self, resource_value: int) -> RarityParameterValue:
        self.resource_value = resource_value
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RarityParameterValue]:
        if data is None:
            return None
        return RarityParameterValue()\
            .with_name(data.get('name'))\
            .with_resource_name(data.get('resourceName'))\
            .with_resource_value(data.get('resourceValue'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "name": self.name,
            "resourceName": self.resource_name,
            "resourceValue": self.resource_value,
        }


class BalanceParameterValue(core.Gs2Model):
    name: str = None
    value: int = None

    def with_name(self, name: str) -> BalanceParameterValue:
        self.name = name
        return self

    def with_value(self, value: int) -> BalanceParameterValue:
        self.value = value
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[BalanceParameterValue]:
        if data is None:
            return None
        return BalanceParameterValue()\
            .with_name(data.get('name'))\
            .with_value(data.get('value'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "name": self.name,
            "value": self.value,
        }


class RarityParameterValueModel(core.Gs2Model):
    name: str = None
    metadata: str = None
    resource_name: str = None
    resource_value: int = None
    weight: int = None

    def with_name(self, name: str) -> RarityParameterValueModel:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> RarityParameterValueModel:
        self.metadata = metadata
        return self

    def with_resource_name(self, resource_name: str) -> RarityParameterValueModel:
        self.resource_name = resource_name
        return self

    def with_resource_value(self, resource_value: int) -> RarityParameterValueModel:
        self.resource_value = resource_value
        return self

    def with_weight(self, weight: int) -> RarityParameterValueModel:
        self.weight = weight
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RarityParameterValueModel]:
        if data is None:
            return None
        return RarityParameterValueModel()\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_resource_name(data.get('resourceName'))\
            .with_resource_value(data.get('resourceValue'))\
            .with_weight(data.get('weight'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "name": self.name,
            "metadata": self.metadata,
            "resourceName": self.resource_name,
            "resourceValue": self.resource_value,
            "weight": self.weight,
        }


class RarityParameterCountModel(core.Gs2Model):
    count: int = None
    weight: int = None

    def with_count(self, count: int) -> RarityParameterCountModel:
        self.count = count
        return self

    def with_weight(self, weight: int) -> RarityParameterCountModel:
        self.weight = weight
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RarityParameterCountModel]:
        if data is None:
            return None
        return RarityParameterCountModel()\
            .with_count(data.get('count'))\
            .with_weight(data.get('weight'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "count": self.count,
            "weight": self.weight,
        }


class BalanceParameterValueModel(core.Gs2Model):
    name: str = None
    metadata: str = None

    def with_name(self, name: str) -> BalanceParameterValueModel:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> BalanceParameterValueModel:
        self.metadata = metadata
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[BalanceParameterValueModel]:
        if data is None:
            return None
        return BalanceParameterValueModel()\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "name": self.name,
            "metadata": self.metadata,
        }


class RarityParameterStatus(core.Gs2Model):
    rarity_parameter_status_id: str = None
    user_id: str = None
    parameter_name: str = None
    property_id: str = None
    parameter_values: List[RarityParameterValue] = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_rarity_parameter_status_id(self, rarity_parameter_status_id: str) -> RarityParameterStatus:
        self.rarity_parameter_status_id = rarity_parameter_status_id
        return self

    def with_user_id(self, user_id: str) -> RarityParameterStatus:
        self.user_id = user_id
        return self

    def with_parameter_name(self, parameter_name: str) -> RarityParameterStatus:
        self.parameter_name = parameter_name
        return self

    def with_property_id(self, property_id: str) -> RarityParameterStatus:
        self.property_id = property_id
        return self

    def with_parameter_values(self, parameter_values: List[RarityParameterValue]) -> RarityParameterStatus:
        self.parameter_values = parameter_values
        return self

    def with_created_at(self, created_at: int) -> RarityParameterStatus:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> RarityParameterStatus:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> RarityParameterStatus:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        user_id,
        parameter_name,
        property_id,
    ):
        return 'grn:gs2:{region}:{ownerId}:enchant:{namespaceName}:user:{userId}:rarity:{parameterName}:{propertyId}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            userId=user_id,
            parameterName=parameter_name,
            propertyId=property_id,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):user:(?P<userId>.+):rarity:(?P<parameterName>.+):(?P<propertyId>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):user:(?P<userId>.+):rarity:(?P<parameterName>.+):(?P<propertyId>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):user:(?P<userId>.+):rarity:(?P<parameterName>.+):(?P<propertyId>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_user_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):user:(?P<userId>.+):rarity:(?P<parameterName>.+):(?P<propertyId>.+)', grn)
        if match is None:
            return None
        return match.group('user_id')

    @classmethod
    def get_parameter_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):user:(?P<userId>.+):rarity:(?P<parameterName>.+):(?P<propertyId>.+)', grn)
        if match is None:
            return None
        return match.group('parameter_name')

    @classmethod
    def get_property_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):user:(?P<userId>.+):rarity:(?P<parameterName>.+):(?P<propertyId>.+)', grn)
        if match is None:
            return None
        return match.group('property_id')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RarityParameterStatus]:
        if data is None:
            return None
        return RarityParameterStatus()\
            .with_rarity_parameter_status_id(data.get('rarityParameterStatusId'))\
            .with_user_id(data.get('userId'))\
            .with_parameter_name(data.get('parameterName'))\
            .with_property_id(data.get('propertyId'))\
            .with_parameter_values(None if data.get('parameterValues') is None else [
                RarityParameterValue.from_dict(data.get('parameterValues')[i])
                for i in range(len(data.get('parameterValues')))
            ])\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "rarityParameterStatusId": self.rarity_parameter_status_id,
            "userId": self.user_id,
            "parameterName": self.parameter_name,
            "propertyId": self.property_id,
            "parameterValues": None if self.parameter_values is None else [
                self.parameter_values[i].to_dict() if self.parameter_values[i] else None
                for i in range(len(self.parameter_values))
            ],
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class BalanceParameterStatus(core.Gs2Model):
    balance_parameter_status_id: str = None
    user_id: str = None
    parameter_name: str = None
    property_id: str = None
    parameter_values: List[BalanceParameterValue] = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_balance_parameter_status_id(self, balance_parameter_status_id: str) -> BalanceParameterStatus:
        self.balance_parameter_status_id = balance_parameter_status_id
        return self

    def with_user_id(self, user_id: str) -> BalanceParameterStatus:
        self.user_id = user_id
        return self

    def with_parameter_name(self, parameter_name: str) -> BalanceParameterStatus:
        self.parameter_name = parameter_name
        return self

    def with_property_id(self, property_id: str) -> BalanceParameterStatus:
        self.property_id = property_id
        return self

    def with_parameter_values(self, parameter_values: List[BalanceParameterValue]) -> BalanceParameterStatus:
        self.parameter_values = parameter_values
        return self

    def with_created_at(self, created_at: int) -> BalanceParameterStatus:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> BalanceParameterStatus:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> BalanceParameterStatus:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        user_id,
        parameter_name,
        property_id,
    ):
        return 'grn:gs2:{region}:{ownerId}:enchant:{namespaceName}:user:{userId}:balance:{parameterName}:{propertyId}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            userId=user_id,
            parameterName=parameter_name,
            propertyId=property_id,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):user:(?P<userId>.+):balance:(?P<parameterName>.+):(?P<propertyId>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):user:(?P<userId>.+):balance:(?P<parameterName>.+):(?P<propertyId>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):user:(?P<userId>.+):balance:(?P<parameterName>.+):(?P<propertyId>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_user_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):user:(?P<userId>.+):balance:(?P<parameterName>.+):(?P<propertyId>.+)', grn)
        if match is None:
            return None
        return match.group('user_id')

    @classmethod
    def get_parameter_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):user:(?P<userId>.+):balance:(?P<parameterName>.+):(?P<propertyId>.+)', grn)
        if match is None:
            return None
        return match.group('parameter_name')

    @classmethod
    def get_property_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):user:(?P<userId>.+):balance:(?P<parameterName>.+):(?P<propertyId>.+)', grn)
        if match is None:
            return None
        return match.group('property_id')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[BalanceParameterStatus]:
        if data is None:
            return None
        return BalanceParameterStatus()\
            .with_balance_parameter_status_id(data.get('balanceParameterStatusId'))\
            .with_user_id(data.get('userId'))\
            .with_parameter_name(data.get('parameterName'))\
            .with_property_id(data.get('propertyId'))\
            .with_parameter_values(None if data.get('parameterValues') is None else [
                BalanceParameterValue.from_dict(data.get('parameterValues')[i])
                for i in range(len(data.get('parameterValues')))
            ])\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "balanceParameterStatusId": self.balance_parameter_status_id,
            "userId": self.user_id,
            "parameterName": self.parameter_name,
            "propertyId": self.property_id,
            "parameterValues": None if self.parameter_values is None else [
                self.parameter_values[i].to_dict() if self.parameter_values[i] else None
                for i in range(len(self.parameter_values))
            ],
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class CurrentParameterMaster(core.Gs2Model):
    namespace_id: str = None
    settings: str = None

    def with_namespace_id(self, namespace_id: str) -> CurrentParameterMaster:
        self.namespace_id = namespace_id
        return self

    def with_settings(self, settings: str) -> CurrentParameterMaster:
        self.settings = settings
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:enchant:{namespaceName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CurrentParameterMaster]:
        if data is None:
            return None
        return CurrentParameterMaster()\
            .with_namespace_id(data.get('namespaceId'))\
            .with_settings(data.get('settings'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "namespaceId": self.namespace_id,
            "settings": self.settings,
        }


class RarityParameterModelMaster(core.Gs2Model):
    rarity_parameter_model_id: str = None
    name: str = None
    description: str = None
    metadata: str = None
    maximum_parameter_count: int = None
    parameter_counts: List[RarityParameterCountModel] = None
    parameters: List[RarityParameterValueModel] = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_rarity_parameter_model_id(self, rarity_parameter_model_id: str) -> RarityParameterModelMaster:
        self.rarity_parameter_model_id = rarity_parameter_model_id
        return self

    def with_name(self, name: str) -> RarityParameterModelMaster:
        self.name = name
        return self

    def with_description(self, description: str) -> RarityParameterModelMaster:
        self.description = description
        return self

    def with_metadata(self, metadata: str) -> RarityParameterModelMaster:
        self.metadata = metadata
        return self

    def with_maximum_parameter_count(self, maximum_parameter_count: int) -> RarityParameterModelMaster:
        self.maximum_parameter_count = maximum_parameter_count
        return self

    def with_parameter_counts(self, parameter_counts: List[RarityParameterCountModel]) -> RarityParameterModelMaster:
        self.parameter_counts = parameter_counts
        return self

    def with_parameters(self, parameters: List[RarityParameterValueModel]) -> RarityParameterModelMaster:
        self.parameters = parameters
        return self

    def with_created_at(self, created_at: int) -> RarityParameterModelMaster:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> RarityParameterModelMaster:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> RarityParameterModelMaster:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        parameter_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:enchant:{namespaceName}:model:rarity:{parameterName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            parameterName=parameter_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:rarity:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:rarity:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:rarity:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_parameter_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:rarity:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('parameter_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RarityParameterModelMaster]:
        if data is None:
            return None
        return RarityParameterModelMaster()\
            .with_rarity_parameter_model_id(data.get('rarityParameterModelId'))\
            .with_name(data.get('name'))\
            .with_description(data.get('description'))\
            .with_metadata(data.get('metadata'))\
            .with_maximum_parameter_count(data.get('maximumParameterCount'))\
            .with_parameter_counts(None if data.get('parameterCounts') is None else [
                RarityParameterCountModel.from_dict(data.get('parameterCounts')[i])
                for i in range(len(data.get('parameterCounts')))
            ])\
            .with_parameters(None if data.get('parameters') is None else [
                RarityParameterValueModel.from_dict(data.get('parameters')[i])
                for i in range(len(data.get('parameters')))
            ])\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "rarityParameterModelId": self.rarity_parameter_model_id,
            "name": self.name,
            "description": self.description,
            "metadata": self.metadata,
            "maximumParameterCount": self.maximum_parameter_count,
            "parameterCounts": None if self.parameter_counts is None else [
                self.parameter_counts[i].to_dict() if self.parameter_counts[i] else None
                for i in range(len(self.parameter_counts))
            ],
            "parameters": None if self.parameters is None else [
                self.parameters[i].to_dict() if self.parameters[i] else None
                for i in range(len(self.parameters))
            ],
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class RarityParameterModel(core.Gs2Model):
    rarity_parameter_model_id: str = None
    name: str = None
    metadata: str = None
    maximum_parameter_count: int = None
    parameter_counts: List[RarityParameterCountModel] = None
    parameters: List[RarityParameterValueModel] = None

    def with_rarity_parameter_model_id(self, rarity_parameter_model_id: str) -> RarityParameterModel:
        self.rarity_parameter_model_id = rarity_parameter_model_id
        return self

    def with_name(self, name: str) -> RarityParameterModel:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> RarityParameterModel:
        self.metadata = metadata
        return self

    def with_maximum_parameter_count(self, maximum_parameter_count: int) -> RarityParameterModel:
        self.maximum_parameter_count = maximum_parameter_count
        return self

    def with_parameter_counts(self, parameter_counts: List[RarityParameterCountModel]) -> RarityParameterModel:
        self.parameter_counts = parameter_counts
        return self

    def with_parameters(self, parameters: List[RarityParameterValueModel]) -> RarityParameterModel:
        self.parameters = parameters
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        parameter_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:enchant:{namespaceName}:model:rarity:{parameterName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            parameterName=parameter_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:rarity:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:rarity:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:rarity:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_parameter_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:rarity:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('parameter_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[RarityParameterModel]:
        if data is None:
            return None
        return RarityParameterModel()\
            .with_rarity_parameter_model_id(data.get('rarityParameterModelId'))\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_maximum_parameter_count(data.get('maximumParameterCount'))\
            .with_parameter_counts(None if data.get('parameterCounts') is None else [
                RarityParameterCountModel.from_dict(data.get('parameterCounts')[i])
                for i in range(len(data.get('parameterCounts')))
            ])\
            .with_parameters(None if data.get('parameters') is None else [
                RarityParameterValueModel.from_dict(data.get('parameters')[i])
                for i in range(len(data.get('parameters')))
            ])

    def to_dict(self) -> Dict[str, Any]:
        return {
            "rarityParameterModelId": self.rarity_parameter_model_id,
            "name": self.name,
            "metadata": self.metadata,
            "maximumParameterCount": self.maximum_parameter_count,
            "parameterCounts": None if self.parameter_counts is None else [
                self.parameter_counts[i].to_dict() if self.parameter_counts[i] else None
                for i in range(len(self.parameter_counts))
            ],
            "parameters": None if self.parameters is None else [
                self.parameters[i].to_dict() if self.parameters[i] else None
                for i in range(len(self.parameters))
            ],
        }


class BalanceParameterModelMaster(core.Gs2Model):
    balance_parameter_model_id: str = None
    name: str = None
    description: str = None
    metadata: str = None
    total_value: int = None
    initial_value_strategy: str = None
    parameters: List[BalanceParameterValueModel] = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_balance_parameter_model_id(self, balance_parameter_model_id: str) -> BalanceParameterModelMaster:
        self.balance_parameter_model_id = balance_parameter_model_id
        return self

    def with_name(self, name: str) -> BalanceParameterModelMaster:
        self.name = name
        return self

    def with_description(self, description: str) -> BalanceParameterModelMaster:
        self.description = description
        return self

    def with_metadata(self, metadata: str) -> BalanceParameterModelMaster:
        self.metadata = metadata
        return self

    def with_total_value(self, total_value: int) -> BalanceParameterModelMaster:
        self.total_value = total_value
        return self

    def with_initial_value_strategy(self, initial_value_strategy: str) -> BalanceParameterModelMaster:
        self.initial_value_strategy = initial_value_strategy
        return self

    def with_parameters(self, parameters: List[BalanceParameterValueModel]) -> BalanceParameterModelMaster:
        self.parameters = parameters
        return self

    def with_created_at(self, created_at: int) -> BalanceParameterModelMaster:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> BalanceParameterModelMaster:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> BalanceParameterModelMaster:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        parameter_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:enchant:{namespaceName}:model:balance:{parameterName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            parameterName=parameter_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:balance:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:balance:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:balance:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_parameter_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:balance:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('parameter_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[BalanceParameterModelMaster]:
        if data is None:
            return None
        return BalanceParameterModelMaster()\
            .with_balance_parameter_model_id(data.get('balanceParameterModelId'))\
            .with_name(data.get('name'))\
            .with_description(data.get('description'))\
            .with_metadata(data.get('metadata'))\
            .with_total_value(data.get('totalValue'))\
            .with_initial_value_strategy(data.get('initialValueStrategy'))\
            .with_parameters(None if data.get('parameters') is None else [
                BalanceParameterValueModel.from_dict(data.get('parameters')[i])
                for i in range(len(data.get('parameters')))
            ])\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "balanceParameterModelId": self.balance_parameter_model_id,
            "name": self.name,
            "description": self.description,
            "metadata": self.metadata,
            "totalValue": self.total_value,
            "initialValueStrategy": self.initial_value_strategy,
            "parameters": None if self.parameters is None else [
                self.parameters[i].to_dict() if self.parameters[i] else None
                for i in range(len(self.parameters))
            ],
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class BalanceParameterModel(core.Gs2Model):
    balance_parameter_model_id: str = None
    name: str = None
    metadata: str = None
    total_value: int = None
    initial_value_strategy: str = None
    parameters: List[BalanceParameterValueModel] = None

    def with_balance_parameter_model_id(self, balance_parameter_model_id: str) -> BalanceParameterModel:
        self.balance_parameter_model_id = balance_parameter_model_id
        return self

    def with_name(self, name: str) -> BalanceParameterModel:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> BalanceParameterModel:
        self.metadata = metadata
        return self

    def with_total_value(self, total_value: int) -> BalanceParameterModel:
        self.total_value = total_value
        return self

    def with_initial_value_strategy(self, initial_value_strategy: str) -> BalanceParameterModel:
        self.initial_value_strategy = initial_value_strategy
        return self

    def with_parameters(self, parameters: List[BalanceParameterValueModel]) -> BalanceParameterModel:
        self.parameters = parameters
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        parameter_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:enchant:{namespaceName}:model:balance:{parameterName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            parameterName=parameter_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:balance:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:balance:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:balance:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_parameter_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+):model:balance:(?P<parameterName>.+)', grn)
        if match is None:
            return None
        return match.group('parameter_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[BalanceParameterModel]:
        if data is None:
            return None
        return BalanceParameterModel()\
            .with_balance_parameter_model_id(data.get('balanceParameterModelId'))\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_total_value(data.get('totalValue'))\
            .with_initial_value_strategy(data.get('initialValueStrategy'))\
            .with_parameters(None if data.get('parameters') is None else [
                BalanceParameterValueModel.from_dict(data.get('parameters')[i])
                for i in range(len(data.get('parameters')))
            ])

    def to_dict(self) -> Dict[str, Any]:
        return {
            "balanceParameterModelId": self.balance_parameter_model_id,
            "name": self.name,
            "metadata": self.metadata,
            "totalValue": self.total_value,
            "initialValueStrategy": self.initial_value_strategy,
            "parameters": None if self.parameters is None else [
                self.parameters[i].to_dict() if self.parameters[i] else None
                for i in range(len(self.parameters))
            ],
        }


class Namespace(core.Gs2Model):
    namespace_id: str = None
    name: str = None
    description: str = None
    transaction_setting: TransactionSetting = None
    log_setting: LogSetting = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_namespace_id(self, namespace_id: str) -> Namespace:
        self.namespace_id = namespace_id
        return self

    def with_name(self, name: str) -> Namespace:
        self.name = name
        return self

    def with_description(self, description: str) -> Namespace:
        self.description = description
        return self

    def with_transaction_setting(self, transaction_setting: TransactionSetting) -> Namespace:
        self.transaction_setting = transaction_setting
        return self

    def with_log_setting(self, log_setting: LogSetting) -> Namespace:
        self.log_setting = log_setting
        return self

    def with_created_at(self, created_at: int) -> Namespace:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> Namespace:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> Namespace:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:enchant:{namespaceName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):enchant:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Namespace]:
        if data is None:
            return None
        return Namespace()\
            .with_namespace_id(data.get('namespaceId'))\
            .with_name(data.get('name'))\
            .with_description(data.get('description'))\
            .with_transaction_setting(TransactionSetting.from_dict(data.get('transactionSetting')))\
            .with_log_setting(LogSetting.from_dict(data.get('logSetting')))\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "namespaceId": self.namespace_id,
            "name": self.name,
            "description": self.description,
            "transactionSetting": self.transaction_setting.to_dict() if self.transaction_setting else None,
            "logSetting": self.log_setting.to_dict() if self.log_setting else None,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

from __future__ import annotations

import re
from typing import *
from gs2 import core


class TransactionSetting(core.Gs2Model):
    enable_auto_run: bool = None
    enable_atomic_commit: bool = None
    transaction_use_distributor: bool = None
    acquire_action_use_job_queue: bool = None
    distributor_namespace_id: str = None
    key_id: str = None
    queue_namespace_id: str = None

    def with_enable_auto_run(self, enable_auto_run: bool) -> TransactionSetting:
        self.enable_auto_run = enable_auto_run
        return self

    def with_enable_atomic_commit(self, enable_atomic_commit: bool) -> TransactionSetting:
        self.enable_atomic_commit = enable_atomic_commit
        return self

    def with_transaction_use_distributor(self, transaction_use_distributor: bool) -> TransactionSetting:
        self.transaction_use_distributor = transaction_use_distributor
        return self

    def with_acquire_action_use_job_queue(self, acquire_action_use_job_queue: bool) -> TransactionSetting:
        self.acquire_action_use_job_queue = acquire_action_use_job_queue
        return self

    def with_distributor_namespace_id(self, distributor_namespace_id: str) -> TransactionSetting:
        self.distributor_namespace_id = distributor_namespace_id
        return self

    def with_key_id(self, key_id: str) -> TransactionSetting:
        self.key_id = key_id
        return self

    def with_queue_namespace_id(self, queue_namespace_id: str) -> TransactionSetting:
        self.queue_namespace_id = queue_namespace_id
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[TransactionSetting]:
        if data is None:
            return None
        return TransactionSetting()\
            .with_enable_auto_run(data.get('enableAutoRun'))\
            .with_enable_atomic_commit(data.get('enableAtomicCommit'))\
            .with_transaction_use_distributor(data.get('transactionUseDistributor'))\
            .with_acquire_action_use_job_queue(data.get('acquireActionUseJobQueue'))\
            .with_distributor_namespace_id(data.get('distributorNamespaceId'))\
            .with_key_id(data.get('keyId'))\
            .with_queue_namespace_id(data.get('queueNamespaceId'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "enableAutoRun": self.enable_auto_run,
            "enableAtomicCommit": self.enable_atomic_commit,
            "transactionUseDistributor": self.transaction_use_distributor,
            "acquireActionUseJobQueue": self.acquire_action_use_job_queue,
            "distributorNamespaceId": self.distributor_namespace_id,
            "keyId": self.key_id,
            "queueNamespaceId": self.queue_namespace_id,
        }


class LogSetting(core.Gs2Model):
    logging_namespace_id: str = None

    def with_logging_namespace_id(self, logging_namespace_id: str) -> LogSetting:
        self.logging_namespace_id = logging_namespace_id
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[LogSetting]:
        if data is None:
            return None
        return LogSetting()\
            .with_logging_namespace_id(data.get('loggingNamespaceId'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "loggingNamespaceId": self.logging_namespace_id,
        }


class GitHubCheckoutSetting(core.Gs2Model):
    api_key_id: str = None
    repository_name: str = None
    source_path: str = None
    reference_type: str = None
    commit_hash: str = None
    branch_name: str = None
    tag_name: str = None

    def with_api_key_id(self, api_key_id: str) -> GitHubCheckoutSetting:
        self.api_key_id = api_key_id
        return self

    def with_repository_name(self, repository_name: str) -> GitHubCheckoutSetting:
        self.repository_name = repository_name
        return self

    def with_source_path(self, source_path: str) -> GitHubCheckoutSetting:
        self.source_path = source_path
        return self

    def with_reference_type(self, reference_type: str) -> GitHubCheckoutSetting:
        self.reference_type = reference_type
        return self

    def with_commit_hash(self, commit_hash: str) -> GitHubCheckoutSetting:
        self.commit_hash = commit_hash
        return self

    def with_branch_name(self, branch_name: str) -> GitHubCheckoutSetting:
        self.branch_name = branch_name
        return self

    def with_tag_name(self, tag_name: str) -> GitHubCheckoutSetting:
        self.tag_name = tag_name
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GitHubCheckoutSetting]:
        if data is None:
            return None
        return GitHubCheckoutSetting()\
            .with_api_key_id(data.get('apiKeyId'))\
            .with_repository_name(data.get('repositoryName'))\
            .with_source_path(data.get('sourcePath'))\
            .with_reference_type(data.get('referenceType'))\
            .with_commit_hash(data.get('commitHash'))\
            .with_branch_name(data.get('branchName'))\
            .with_tag_name(data.get('tagName'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "apiKeyId": self.api_key_id,
            "repositoryName": self.repository_name,
            "sourcePath": self.source_path,
            "referenceType": self.reference_type,
            "commitHash": self.commit_hash,
            "branchName": self.branch_name,
            "tagName": self.tag_name,
        }


class NotificationSetting(core.Gs2Model):
    gateway_namespace_id: str = None
    enable_transfer_mobile_notification: bool = None
    sound: str = None

    def with_gateway_namespace_id(self, gateway_namespace_id: str) -> NotificationSetting:
        self.gateway_namespace_id = gateway_namespace_id
        return self

    def with_enable_transfer_mobile_notification(self, enable_transfer_mobile_notification: bool) -> NotificationSetting:
        self.enable_transfer_mobile_notification = enable_transfer_mobile_notification
        return self

    def with_sound(self, sound: str) -> NotificationSetting:
        self.sound = sound
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[NotificationSetting]:
        if data is None:
            return None
        return NotificationSetting()\
            .with_gateway_namespace_id(data.get('gatewayNamespaceId'))\
            .with_enable_transfer_mobile_notification(data.get('enableTransferMobileNotification'))\
            .with_sound(data.get('sound'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "gatewayNamespaceId": self.gateway_namespace_id,
            "enableTransferMobileNotification": self.enable_transfer_mobile_notification,
            "sound": self.sound,
        }


class ScriptSetting(core.Gs2Model):
    trigger_script_id: str = None
    done_trigger_target_type: str = None
    done_trigger_script_id: str = None
    done_trigger_queue_namespace_id: str = None

    def with_trigger_script_id(self, trigger_script_id: str) -> ScriptSetting:
        self.trigger_script_id = trigger_script_id
        return self

    def with_done_trigger_target_type(self, done_trigger_target_type: str) -> ScriptSetting:
        self.done_trigger_target_type = done_trigger_target_type
        return self

    def with_done_trigger_script_id(self, done_trigger_script_id: str) -> ScriptSetting:
        self.done_trigger_script_id = done_trigger_script_id
        return self

    def with_done_trigger_queue_namespace_id(self, done_trigger_queue_namespace_id: str) -> ScriptSetting:
        self.done_trigger_queue_namespace_id = done_trigger_queue_namespace_id
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[ScriptSetting]:
        if data is None:
            return None
        return ScriptSetting()\
            .with_trigger_script_id(data.get('triggerScriptId'))\
            .with_done_trigger_target_type(data.get('doneTriggerTargetType'))\
            .with_done_trigger_script_id(data.get('doneTriggerScriptId'))\
            .with_done_trigger_queue_namespace_id(data.get('doneTriggerQueueNamespaceId'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "triggerScriptId": self.trigger_script_id,
            "doneTriggerTargetType": self.done_trigger_target_type,
            "doneTriggerScriptId": self.done_trigger_script_id,
            "doneTriggerQueueNamespaceId": self.done_trigger_queue_namespace_id,
        }


class Config(core.Gs2Model):
    key: str = None
    value: str = None

    def with_key(self, key: str) -> Config:
        self.key = key
        return self

    def with_value(self, value: str) -> Config:
        self.value = value
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Config]:
        if data is None:
            return None
        return Config()\
            .with_key(data.get('key'))\
            .with_value(data.get('value'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "key": self.key,
            "value": self.value,
        }


class AcquireAction(core.Gs2Model):
    action: str = None
    request: str = None

    def with_action(self, action: str) -> AcquireAction:
        self.action = action
        return self

    def with_request(self, request: str) -> AcquireAction:
        self.request = request
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[AcquireAction]:
        if data is None:
            return None
        return AcquireAction()\
            .with_action(data.get('action'))\
            .with_request(data.get('request'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "action": self.action,
            "request": self.request,
        }


class TimeSpan(core.Gs2Model):
    days: int = None
    hours: int = None
    minutes: int = None

    def with_days(self, days: int) -> TimeSpan:
        self.days = days
        return self

    def with_hours(self, hours: int) -> TimeSpan:
        self.hours = hours
        return self

    def with_minutes(self, minutes: int) -> TimeSpan:
        self.minutes = minutes
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[TimeSpan]:
        if data is None:
            return None
        return TimeSpan()\
            .with_days(data.get('days'))\
            .with_hours(data.get('hours'))\
            .with_minutes(data.get('minutes'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "days": self.days,
            "hours": self.hours,
            "minutes": self.minutes,
        }


class Received(core.Gs2Model):
    received_id: str = None
    user_id: str = None
    received_global_message_names: List[str] = None
    created_at: int = None
    updated_at: int = None
    revision: int = None

    def with_received_id(self, received_id: str) -> Received:
        self.received_id = received_id
        return self

    def with_user_id(self, user_id: str) -> Received:
        self.user_id = user_id
        return self

    def with_received_global_message_names(self, received_global_message_names: List[str]) -> Received:
        self.received_global_message_names = received_global_message_names
        return self

    def with_created_at(self, created_at: int) -> Received:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> Received:
        self.updated_at = updated_at
        return self

    def with_revision(self, revision: int) -> Received:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        user_id,
    ):
        return 'grn:gs2:{region}:{ownerId}:inbox:{namespaceName}:user:{userId}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            userId=user_id,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):user:(?P<userId>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):user:(?P<userId>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):user:(?P<userId>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_user_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):user:(?P<userId>.+)', grn)
        if match is None:
            return None
        return match.group('user_id')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Received]:
        if data is None:
            return None
        return Received()\
            .with_received_id(data.get('receivedId'))\
            .with_user_id(data.get('userId'))\
            .with_received_global_message_names(None if data.get('receivedGlobalMessageNames') is None else [
                data.get('receivedGlobalMessageNames')[i]
                for i in range(len(data.get('receivedGlobalMessageNames')))
            ])\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "receivedId": self.received_id,
            "userId": self.user_id,
            "receivedGlobalMessageNames": None if self.received_global_message_names is None else [
                self.received_global_message_names[i]
                for i in range(len(self.received_global_message_names))
            ],
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "revision": self.revision,
        }


class GlobalMessage(core.Gs2Model):
    global_message_id: str = None
    name: str = None
    metadata: str = None
    read_acquire_actions: List[AcquireAction] = None
    expires_time_span: TimeSpan = None
    expires_at: int = None
    message_reception_period_event_id: str = None

    def with_global_message_id(self, global_message_id: str) -> GlobalMessage:
        self.global_message_id = global_message_id
        return self

    def with_name(self, name: str) -> GlobalMessage:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> GlobalMessage:
        self.metadata = metadata
        return self

    def with_read_acquire_actions(self, read_acquire_actions: List[AcquireAction]) -> GlobalMessage:
        self.read_acquire_actions = read_acquire_actions
        return self

    def with_expires_time_span(self, expires_time_span: TimeSpan) -> GlobalMessage:
        self.expires_time_span = expires_time_span
        return self

    def with_expires_at(self, expires_at: int) -> GlobalMessage:
        self.expires_at = expires_at
        return self

    def with_message_reception_period_event_id(self, message_reception_period_event_id: str) -> GlobalMessage:
        self.message_reception_period_event_id = message_reception_period_event_id
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        global_message_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:inbox:{namespaceName}:globalMessage:{globalMessageName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            globalMessageName=global_message_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):globalMessage:(?P<globalMessageName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):globalMessage:(?P<globalMessageName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):globalMessage:(?P<globalMessageName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_global_message_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):globalMessage:(?P<globalMessageName>.+)', grn)
        if match is None:
            return None
        return match.group('global_message_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GlobalMessage]:
        if data is None:
            return None
        return GlobalMessage()\
            .with_global_message_id(data.get('globalMessageId'))\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_read_acquire_actions(None if data.get('readAcquireActions') is None else [
                AcquireAction.from_dict(data.get('readAcquireActions')[i])
                for i in range(len(data.get('readAcquireActions')))
            ])\
            .with_expires_time_span(TimeSpan.from_dict(data.get('expiresTimeSpan')))\
            .with_expires_at(data.get('expiresAt'))\
            .with_message_reception_period_event_id(data.get('messageReceptionPeriodEventId'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "globalMessageId": self.global_message_id,
            "name": self.name,
            "metadata": self.metadata,
            "readAcquireActions": None if self.read_acquire_actions is None else [
                self.read_acquire_actions[i].to_dict() if self.read_acquire_actions[i] else None
                for i in range(len(self.read_acquire_actions))
            ],
            "expiresTimeSpan": self.expires_time_span.to_dict() if self.expires_time_span else None,
            "expiresAt": self.expires_at,
            "messageReceptionPeriodEventId": self.message_reception_period_event_id,
        }


class GlobalMessageMaster(core.Gs2Model):
    global_message_id: str = None
    name: str = None
    metadata: str = None
    read_acquire_actions: List[AcquireAction] = None
    expires_time_span: TimeSpan = None
    expires_at: int = None
    message_reception_period_event_id: str = None
    created_at: int = None
    revision: int = None

    def with_global_message_id(self, global_message_id: str) -> GlobalMessageMaster:
        self.global_message_id = global_message_id
        return self

    def with_name(self, name: str) -> GlobalMessageMaster:
        self.name = name
        return self

    def with_metadata(self, metadata: str) -> GlobalMessageMaster:
        self.metadata = metadata
        return self

    def with_read_acquire_actions(self, read_acquire_actions: List[AcquireAction]) -> GlobalMessageMaster:
        self.read_acquire_actions = read_acquire_actions
        return self

    def with_expires_time_span(self, expires_time_span: TimeSpan) -> GlobalMessageMaster:
        self.expires_time_span = expires_time_span
        return self

    def with_expires_at(self, expires_at: int) -> GlobalMessageMaster:
        self.expires_at = expires_at
        return self

    def with_message_reception_period_event_id(self, message_reception_period_event_id: str) -> GlobalMessageMaster:
        self.message_reception_period_event_id = message_reception_period_event_id
        return self

    def with_created_at(self, created_at: int) -> GlobalMessageMaster:
        self.created_at = created_at
        return self

    def with_revision(self, revision: int) -> GlobalMessageMaster:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        global_message_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:inbox:{namespaceName}:master:globalMessage:{globalMessageName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            globalMessageName=global_message_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):master:globalMessage:(?P<globalMessageName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):master:globalMessage:(?P<globalMessageName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):master:globalMessage:(?P<globalMessageName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_global_message_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):master:globalMessage:(?P<globalMessageName>.+)', grn)
        if match is None:
            return None
        return match.group('global_message_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GlobalMessageMaster]:
        if data is None:
            return None
        return GlobalMessageMaster()\
            .with_global_message_id(data.get('globalMessageId'))\
            .with_name(data.get('name'))\
            .with_metadata(data.get('metadata'))\
            .with_read_acquire_actions(None if data.get('readAcquireActions') is None else [
                AcquireAction.from_dict(data.get('readAcquireActions')[i])
                for i in range(len(data.get('readAcquireActions')))
            ])\
            .with_expires_time_span(TimeSpan.from_dict(data.get('expiresTimeSpan')))\
            .with_expires_at(data.get('expiresAt'))\
            .with_message_reception_period_event_id(data.get('messageReceptionPeriodEventId'))\
            .with_created_at(data.get('createdAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "globalMessageId": self.global_message_id,
            "name": self.name,
            "metadata": self.metadata,
            "readAcquireActions": None if self.read_acquire_actions is None else [
                self.read_acquire_actions[i].to_dict() if self.read_acquire_actions[i] else None
                for i in range(len(self.read_acquire_actions))
            ],
            "expiresTimeSpan": self.expires_time_span.to_dict() if self.expires_time_span else None,
            "expiresAt": self.expires_at,
            "messageReceptionPeriodEventId": self.message_reception_period_event_id,
            "createdAt": self.created_at,
            "revision": self.revision,
        }


class CurrentMessageMaster(core.Gs2Model):
    namespace_id: str = None
    settings: str = None

    def with_namespace_id(self, namespace_id: str) -> CurrentMessageMaster:
        self.namespace_id = namespace_id
        return self

    def with_settings(self, settings: str) -> CurrentMessageMaster:
        self.settings = settings
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:inbox:{namespaceName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[CurrentMessageMaster]:
        if data is None:
            return None
        return CurrentMessageMaster()\
            .with_namespace_id(data.get('namespaceId'))\
            .with_settings(data.get('settings'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "namespaceId": self.namespace_id,
            "settings": self.settings,
        }


class Message(core.Gs2Model):
    message_id: str = None
    name: str = None
    user_id: str = None
    metadata: str = None
    is_read: bool = None
    read_acquire_actions: List[AcquireAction] = None
    received_at: int = None
    read_at: int = None
    expires_at: int = None
    revision: int = None

    def with_message_id(self, message_id: str) -> Message:
        self.message_id = message_id
        return self

    def with_name(self, name: str) -> Message:
        self.name = name
        return self

    def with_user_id(self, user_id: str) -> Message:
        self.user_id = user_id
        return self

    def with_metadata(self, metadata: str) -> Message:
        self.metadata = metadata
        return self

    def with_is_read(self, is_read: bool) -> Message:
        self.is_read = is_read
        return self

    def with_read_acquire_actions(self, read_acquire_actions: List[AcquireAction]) -> Message:
        self.read_acquire_actions = read_acquire_actions
        return self

    def with_received_at(self, received_at: int) -> Message:
        self.received_at = received_at
        return self

    def with_read_at(self, read_at: int) -> Message:
        self.read_at = read_at
        return self

    def with_expires_at(self, expires_at: int) -> Message:
        self.expires_at = expires_at
        return self

    def with_revision(self, revision: int) -> Message:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
        user_id,
        message_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:inbox:{namespaceName}:user:{userId}:message:{messageName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
            userId=user_id,
            messageName=message_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):user:(?P<userId>.+):message:(?P<messageName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):user:(?P<userId>.+):message:(?P<messageName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):user:(?P<userId>.+):message:(?P<messageName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    @classmethod
    def get_user_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):user:(?P<userId>.+):message:(?P<messageName>.+)', grn)
        if match is None:
            return None
        return match.group('user_id')

    @classmethod
    def get_message_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+):user:(?P<userId>.+):message:(?P<messageName>.+)', grn)
        if match is None:
            return None
        return match.group('message_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Message]:
        if data is None:
            return None
        return Message()\
            .with_message_id(data.get('messageId'))\
            .with_name(data.get('name'))\
            .with_user_id(data.get('userId'))\
            .with_metadata(data.get('metadata'))\
            .with_is_read(data.get('isRead'))\
            .with_read_acquire_actions(None if data.get('readAcquireActions') is None else [
                AcquireAction.from_dict(data.get('readAcquireActions')[i])
                for i in range(len(data.get('readAcquireActions')))
            ])\
            .with_received_at(data.get('receivedAt'))\
            .with_read_at(data.get('readAt'))\
            .with_expires_at(data.get('expiresAt'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "messageId": self.message_id,
            "name": self.name,
            "userId": self.user_id,
            "metadata": self.metadata,
            "isRead": self.is_read,
            "readAcquireActions": None if self.read_acquire_actions is None else [
                self.read_acquire_actions[i].to_dict() if self.read_acquire_actions[i] else None
                for i in range(len(self.read_acquire_actions))
            ],
            "receivedAt": self.received_at,
            "readAt": self.read_at,
            "expiresAt": self.expires_at,
            "revision": self.revision,
        }


class Namespace(core.Gs2Model):
    namespace_id: str = None
    name: str = None
    description: str = None
    is_automatic_deleting_enabled: bool = None
    transaction_setting: TransactionSetting = None
    receive_message_script: ScriptSetting = None
    read_message_script: ScriptSetting = None
    delete_message_script: ScriptSetting = None
    receive_notification: NotificationSetting = None
    log_setting: LogSetting = None
    created_at: int = None
    updated_at: int = None
    queue_namespace_id: str = None
    key_id: str = None
    revision: int = None

    def with_namespace_id(self, namespace_id: str) -> Namespace:
        self.namespace_id = namespace_id
        return self

    def with_name(self, name: str) -> Namespace:
        self.name = name
        return self

    def with_description(self, description: str) -> Namespace:
        self.description = description
        return self

    def with_is_automatic_deleting_enabled(self, is_automatic_deleting_enabled: bool) -> Namespace:
        self.is_automatic_deleting_enabled = is_automatic_deleting_enabled
        return self

    def with_transaction_setting(self, transaction_setting: TransactionSetting) -> Namespace:
        self.transaction_setting = transaction_setting
        return self

    def with_receive_message_script(self, receive_message_script: ScriptSetting) -> Namespace:
        self.receive_message_script = receive_message_script
        return self

    def with_read_message_script(self, read_message_script: ScriptSetting) -> Namespace:
        self.read_message_script = read_message_script
        return self

    def with_delete_message_script(self, delete_message_script: ScriptSetting) -> Namespace:
        self.delete_message_script = delete_message_script
        return self

    def with_receive_notification(self, receive_notification: NotificationSetting) -> Namespace:
        self.receive_notification = receive_notification
        return self

    def with_log_setting(self, log_setting: LogSetting) -> Namespace:
        self.log_setting = log_setting
        return self

    def with_created_at(self, created_at: int) -> Namespace:
        self.created_at = created_at
        return self

    def with_updated_at(self, updated_at: int) -> Namespace:
        self.updated_at = updated_at
        return self

    def with_queue_namespace_id(self, queue_namespace_id: str) -> Namespace:
        self.queue_namespace_id = queue_namespace_id
        return self

    def with_key_id(self, key_id: str) -> Namespace:
        self.key_id = key_id
        return self

    def with_revision(self, revision: int) -> Namespace:
        self.revision = revision
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        namespace_name,
    ):
        return 'grn:gs2:{region}:{ownerId}:inbox:{namespaceName}'.format(
            region=region,
            ownerId=owner_id,
            namespaceName=namespace_name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_namespace_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):inbox:(?P<namespaceName>.+)', grn)
        if match is None:
            return None
        return match.group('namespace_name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Namespace]:
        if data is None:
            return None
        return Namespace()\
            .with_namespace_id(data.get('namespaceId'))\
            .with_name(data.get('name'))\
            .with_description(data.get('description'))\
            .with_is_automatic_deleting_enabled(data.get('isAutomaticDeletingEnabled'))\
            .with_transaction_setting(TransactionSetting.from_dict(data.get('transactionSetting')))\
            .with_receive_message_script(ScriptSetting.from_dict(data.get('receiveMessageScript')))\
            .with_read_message_script(ScriptSetting.from_dict(data.get('readMessageScript')))\
            .with_delete_message_script(ScriptSetting.from_dict(data.get('deleteMessageScript')))\
            .with_receive_notification(NotificationSetting.from_dict(data.get('receiveNotification')))\
            .with_log_setting(LogSetting.from_dict(data.get('logSetting')))\
            .with_created_at(data.get('createdAt'))\
            .with_updated_at(data.get('updatedAt'))\
            .with_queue_namespace_id(data.get('queueNamespaceId'))\
            .with_key_id(data.get('keyId'))\
            .with_revision(data.get('revision'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "namespaceId": self.namespace_id,
            "name": self.name,
            "description": self.description,
            "isAutomaticDeletingEnabled": self.is_automatic_deleting_enabled,
            "transactionSetting": self.transaction_setting.to_dict() if self.transaction_setting else None,
            "receiveMessageScript": self.receive_message_script.to_dict() if self.receive_message_script else None,
            "readMessageScript": self.read_message_script.to_dict() if self.read_message_script else None,
            "deleteMessageScript": self.delete_message_script.to_dict() if self.delete_message_script else None,
            "receiveNotification": self.receive_notification.to_dict() if self.receive_notification else None,
            "logSetting": self.log_setting.to_dict() if self.log_setting else None,
            "createdAt": self.created_at,
            "updatedAt": self.updated_at,
            "queueNamespaceId": self.queue_namespace_id,
            "keyId": self.key_id,
            "revision": self.revision,
        }